﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DesignPatternsWizard
{
    using System.Windows.Forms;
    using System.Runtime.InteropServices;
    using System.ComponentModel;
    using System.Collections;
    using System.Reflection;
    using System.IO;
    using System.Net;
    using System.Text.RegularExpressions;
    using Microsoft.Win32;
    using System.Diagnostics;

    public interface ITemplateArgument : IServiceProvider
    {
        string ArgumentName { get; }
        string Description { get; }
        string Value { get; set; }
        bool IsRuntime { get; }
    }

    public interface ITabWizard : IDisposable, IServiceProvider
    {
        SortableBindingList<ITemplatePattern> Templates { get; }
        ITemplatePattern SelectedPattern { get; }
    }

    public class SortableBindingList<T> : BindingList<T>
    {
        private readonly Dictionary<Type, PropertyComparer<T>> comparers;
        private bool isSorted;
        private ListSortDirection listSortDirection;
        private PropertyDescriptor propertyDescriptor;
        public class PropertyComparer<U> : IComparer<U>
        {
            private readonly IComparer comparer;
            private PropertyDescriptor propertyDescriptor;
            private int reverse;

            public PropertyComparer(PropertyDescriptor property, ListSortDirection direction)
            {
                this.propertyDescriptor = property;
                Type comparerForPropertyType = typeof(Comparer<>).MakeGenericType(property.PropertyType);
                this.comparer = (IComparer)comparerForPropertyType.InvokeMember("Default", BindingFlags.Static | BindingFlags.GetProperty | BindingFlags.Public, null, null, null);
                this.SetListSortDirection(direction);
            }

            #region IComparer<T> Members

            public int Compare(U x, U y)
            {
                return this.reverse * this.comparer.Compare(this.propertyDescriptor.GetValue(x), this.propertyDescriptor.GetValue(y));
            }

            #endregion

            private void SetPropertyDescriptor(PropertyDescriptor descriptor)
            {
                this.propertyDescriptor = descriptor;
            }

            private void SetListSortDirection(ListSortDirection direction)
            {
                this.reverse = direction == ListSortDirection.Ascending ? 1 : -1;
            }

            public void SetPropertyAndDirection(PropertyDescriptor descriptor, ListSortDirection direction)
            {
                this.SetPropertyDescriptor(descriptor);
                this.SetListSortDirection(direction);
            }
        }
        public SortableBindingList()
            : base(new List<T>())
        {
            this.comparers = new Dictionary<Type, PropertyComparer<T>>();
        }

        public SortableBindingList(IEnumerable<T> enumeration)
            : base(new List<T>(enumeration))
        {
            this.comparers = new Dictionary<Type, PropertyComparer<T>>();
        }

        protected override bool SupportsSortingCore
        {
            get { return true; }
        }

        protected override bool IsSortedCore
        {
            get { return this.isSorted; }
        }

        protected override PropertyDescriptor SortPropertyCore
        {
            get { return this.propertyDescriptor; }
        }

        protected override ListSortDirection SortDirectionCore
        {
            get { return this.listSortDirection; }
        }

        protected override bool SupportsSearchingCore
        {
            get { return true; }
        }

        protected override void ApplySortCore(PropertyDescriptor property, ListSortDirection direction)
        {
            List<T> itemsList = (List<T>)this.Items;

            Type propertyType = property.PropertyType;
            PropertyComparer<T> comparer;
            if (!this.comparers.TryGetValue(propertyType, out comparer))
            {
                comparer = new PropertyComparer<T>(property, direction);
                this.comparers.Add(propertyType, comparer);
            }

            comparer.SetPropertyAndDirection(property, direction);
            itemsList.Sort(comparer);

            this.propertyDescriptor = property;
            this.listSortDirection = direction;
            this.isSorted = true;

            this.OnListChanged(new ListChangedEventArgs(ListChangedType.Reset, -1));
        }

        protected override void RemoveSortCore()
        {
            this.isSorted = false;
            this.propertyDescriptor = base.SortPropertyCore;
            this.listSortDirection = base.SortDirectionCore;

            this.OnListChanged(new ListChangedEventArgs(ListChangedType.Reset, -1));
        }

        protected override int FindCore(PropertyDescriptor property, object key)
        {
            int count = this.Count;
            for (int i = 0; i < count; ++i)
            {
                T element = this[i];
                if (property.GetValue(element).Equals(key))
                {
                    return i;
                }
            }

            return -1;
        }
    }
    #region Windows Messages
    //see http://msdn.microsoft.com/en-us/library/windows/desktop/ms646360(v=vs.85).aspx   
    public enum SystemCommands
    {
        SC_SIZE = 0xF000,
        SC_MOVE = 0xF010,
        SC_MINIMIZE = 0xF020,

        ///<summary>    
        /// Sent when form maximizes    
        ///</summary>    
        SC_MAXIMIZE = 0xF030,

        ///<summary>    /// Sent when form maximizes because of doubcle click on caption    
        /// JTE: Don't use this constant. As per the documentation, you    
        ///      must mask off the last 4 bits of wParam by AND'ing it    
        ///      with 0xFFF0. You can't assume the last 4 bits.     
        ///</summary>    
        SC_MAXIMIZE2 = 0xF032,
        SC_NEXTWINDOW = 0xF040,
        SC_PREVWINDOW = 0xF050,
        ///<summary>    
        /// Closes the form    
        ///</summary>    
        SC_CLOSE = 0xF060,
        SC_VSCROLL = 0xF070,
        SC_HSCROLL = 0xF080,
        SC_MOUSEMENU = 0xF090,
        SC_KEYMENU = 0xF100,
        SC_ARRANGE = 0xF110,
        ///<summary>    
        /// Sent when form is maximized from the taskbar    
        ///</summary>    
        SC_RESTORE = 0xF120,
        ///<summary>    
        /// Sent when form maximizes because of doubcle click on caption    
        /// JTE: Don't use this constant. As per the documentation, you    
        ///      must mask off the last 4 bits of wParam by AND'ing it    
        ///      with 0xFFF0. You can't assume the last 4 bits.     
        ///</summary>    
        SC_RESTORE2 = 0xF122,
        SC_TASKLIST = 0xF130,
        SC_SCREENSAVE = 0xF140,
        SC_HOTKEY = 0xF150,
        SC_DEFAULT = 0xF160,
        SC_MONITORPOWER = 0xF170,
        SC_CONTEXTHELP = 0xF180,
        SC_SEPARATOR = 0xF00F
    }
    internal enum Msg
    {
        WM_NULL = 0x0000,
        WM_CREATE = 0x0001,
        WM_DESTROY = 0x0002,
        WM_MOVE = 0x0003,
        WM_SIZE = 0x0005,
        WM_ACTIVATE = 0x0006,
        WM_SETFOCUS = 0x0007,
        WM_KILLFOCUS = 0x0008,
        WM_ENABLE = 0x000A,
        WM_SETREDRAW = 0x000B,
        WM_SETTEXT = 0x000C,
        WM_GETTEXT = 0x000D,
        WM_GETTEXTLENGTH = 0x000E,
        WM_PAINT = 0x000F,
        WM_CLOSE = 0x0010,
        WM_QUERYENDSESSION = 0x0011,
        WM_QUIT = 0x0012,
        WM_QUERYOPEN = 0x0013,
        WM_ERASEBKGND = 0x0014,
        WM_SYSCOLORCHANGE = 0x0015,
        WM_ENDSESSION = 0x0016,
        WM_SHOWWINDOW = 0x0018,
        WM_CTLCOLOR = 0x0019,
        WM_WININICHANGE = 0x001A,
        WM_SETTINGCHANGE = 0x001A,
        WM_DEVMODECHANGE = 0x001B,
        WM_ACTIVATEAPP = 0x001C,
        WM_FONTCHANGE = 0x001D,
        WM_TIMECHANGE = 0x001E,
        WM_CANCELMODE = 0x001F,
        WM_SETCURSOR = 0x0020,
        WM_MOUSEACTIVATE = 0x0021,
        WM_CHILDACTIVATE = 0x0022,
        WM_QUEUESYNC = 0x0023,
        WM_GETMINMAXINFO = 0x0024,
        WM_PAINTICON = 0x0026,
        WM_ICONERASEBKGND = 0x0027,
        WM_NEXTDLGCTL = 0x0028,
        WM_SPOOLERSTATUS = 0x002A,
        WM_DRAWITEM = 0x002B,
        WM_MEASUREITEM = 0x002C,
        WM_DELETEITEM = 0x002D,
        WM_VKEYTOITEM = 0x002E,
        WM_CHARTOITEM = 0x002F,
        WM_SETFONT = 0x0030,
        WM_GETFONT = 0x0031,
        WM_SETHOTKEY = 0x0032,
        WM_GETHOTKEY = 0x0033,
        WM_QUERYDRAGICON = 0x0037,
        WM_COMPAREITEM = 0x0039,
        WM_GETOBJECT = 0x003D,
        WM_COMPACTING = 0x0041,
        WM_COMMNOTIFY = 0x0044,
        WM_WINDOWPOSCHANGING = 0x0046,
        WM_WINDOWPOSCHANGED = 0x0047,
        WM_POWER = 0x0048,
        WM_COPYDATA = 0x004A,
        WM_CANCELJOURNAL = 0x004B,
        WM_NOTIFY = 0x004E,
        WM_INPUTLANGCHANGEREQUEST = 0x0050,
        WM_INPUTLANGCHANGE = 0x0051,
        WM_TCARD = 0x0052,
        WM_HELP = 0x0053,
        WM_USERCHANGED = 0x0054,
        WM_NOTIFYFORMAT = 0x0055,
        WM_CONTEXTMENU = 0x007B,
        WM_STYLECHANGING = 0x007C,
        WM_STYLECHANGED = 0x007D,
        WM_DISPLAYCHANGE = 0x007E,
        WM_GETICON = 0x007F,
        WM_SETICON = 0x0080,
        WM_NCCREATE = 0x0081,
        WM_NCDESTROY = 0x0082,
        WM_NCCALCSIZE = 0x0083,
        WM_NCHITTEST = 0x0084,
        WM_NCPAINT = 0x0085,
        WM_NCACTIVATE = 0x0086,
        WM_GETDLGCODE = 0x0087,
        WM_SYNCPAINT = 0x0088,
        WM_NCMOUSEMOVE = 0x00A0,
        WM_NCLBUTTONDOWN = 0x00A1,
        WM_NCLBUTTONUP = 0x00A2,
        WM_NCLBUTTONDBLCLK = 0x00A3,
        WM_NCRBUTTONDOWN = 0x00A4,
        WM_NCRBUTTONUP = 0x00A5,
        WM_NCRBUTTONDBLCLK = 0x00A6,
        WM_NCMBUTTONDOWN = 0x00A7,
        WM_NCMBUTTONUP = 0x00A8,
        WM_NCMBUTTONDBLCLK = 0x00A9,
        WM_NCXBUTTONDOWN = 0x00AB,
        WM_NCXBUTTONUP = 0x00AC,
        WM_NCXBUTTONDBLCLK = 0x00AD,
        WM_KEYDOWN = 0x0100,
        WM_KEYUP = 0x0101,
        WM_CHAR = 0x0102,
        WM_DEADCHAR = 0x0103,
        WM_SYSKEYDOWN = 0x0104,
        WM_SYSKEYUP = 0x0105,
        WM_SYSCHAR = 0x0106,
        WM_SYSDEADCHAR = 0x0107,
        WM_KEYLAST = 0x0108,
        WM_IME_STARTCOMPOSITION = 0x010D,
        WM_IME_ENDCOMPOSITION = 0x010E,
        WM_IME_COMPOSITION = 0x010F,
        WM_IME_KEYLAST = 0x010F,
        WM_INITDIALOG = 0x0110,
        WM_COMMAND = 0x0111,
        WM_SYSCOMMAND = 0x0112,
        WM_TIMER = 0x0113,
        WM_HSCROLL = 0x0114,
        WM_VSCROLL = 0x0115,
        WM_INITMENU = 0x0116,
        WM_INITMENUPOPUP = 0x0117,
        WM_MENUSELECT = 0x011F,
        WM_MENUCHAR = 0x0120,
        WM_ENTERIDLE = 0x0121,
        WM_MENURBUTTONUP = 0x0122,
        WM_MENUDRAG = 0x0123,
        WM_MENUGETOBJECT = 0x0124,
        WM_UNINITMENUPOPUP = 0x0125,
        WM_MENUCOMMAND = 0x0126,
        WM_CTLCOLORMSGBOX = 0x0132,
        WM_CTLCOLOREDIT = 0x0133,
        WM_CTLCOLORLISTBOX = 0x0134,
        WM_CTLCOLORBTN = 0x0135,
        WM_CTLCOLORDLG = 0x0136,
        WM_CTLCOLORSCROLLBAR = 0x0137,
        WM_CTLCOLORSTATIC = 0x0138,
        WM_MOUSEMOVE = 0x0200,
        WM_LBUTTONDOWN = 0x0201,
        WM_LBUTTONUP = 0x0202,
        WM_LBUTTONDBLCLK = 0x0203,
        WM_RBUTTONDOWN = 0x0204,
        WM_RBUTTONUP = 0x0205,
        WM_RBUTTONDBLCLK = 0x0206,
        WM_MBUTTONDOWN = 0x0207,
        WM_MBUTTONUP = 0x0208,
        WM_MBUTTONDBLCLK = 0x0209,
        WM_MOUSEWHEEL = 0x020A,
        WM_XBUTTONDOWN = 0x020B,
        WM_XBUTTONUP = 0x020C,
        WM_XBUTTONDBLCLK = 0x020D,
        WM_PARENTNOTIFY = 0x0210,
        WM_ENTERMENULOOP = 0x0211,
        WM_EXITMENULOOP = 0x0212,
        WM_NEXTMENU = 0x0213,
        WM_SIZING = 0x0214,
        WM_CAPTURECHANGED = 0x0215,
        WM_MOVING = 0x0216,
        WM_DEVICECHANGE = 0x0219,
        WM_MDICREATE = 0x0220,
        WM_MDIDESTROY = 0x0221,
        WM_MDIACTIVATE = 0x0222,
        WM_MDIRESTORE = 0x0223,
        WM_MDINEXT = 0x0224,
        WM_MDIMAXIMIZE = 0x0225,
        WM_MDITILE = 0x0226,
        WM_MDICASCADE = 0x0227,
        WM_MDIICONARRANGE = 0x0228,
        WM_MDIGETACTIVE = 0x0229,
        WM_MDISETMENU = 0x0230,
        WM_ENTERSIZEMOVE = 0x0231,
        WM_EXITSIZEMOVE = 0x0232,
        WM_DROPFILES = 0x0233,
        WM_MDIREFRESHMENU = 0x0234,
        WM_IME_SETCONTEXT = 0x0281,
        WM_IME_NOTIFY = 0x0282,
        WM_IME_CONTROL = 0x0283,
        WM_IME_COMPOSITIONFULL = 0x0284,
        WM_IME_SELECT = 0x0285,
        WM_IME_CHAR = 0x0286,
        WM_IME_REQUEST = 0x0288,
        WM_IME_KEYDOWN = 0x0290,
        WM_IME_KEYUP = 0x0291,
        WM_MOUSEHOVER = 0x02A1,
        WM_MOUSELEAVE = 0x02A3,
        WM_CUT = 0x0300,
        WM_COPY = 0x0301,
        WM_PASTE = 0x0302,
        WM_CLEAR = 0x0303,
        WM_UNDO = 0x0304,
        WM_RENDERFORMAT = 0x0305,
        WM_RENDERALLFORMATS = 0x0306,
        WM_DESTROYCLIPBOARD = 0x0307,
        WM_DRAWCLIPBOARD = 0x0308,
        WM_PAINTCLIPBOARD = 0x0309,
        WM_VSCROLLCLIPBOARD = 0x030A,
        WM_SIZECLIPBOARD = 0x030B,
        WM_ASKCBFORMATNAME = 0x030C,
        WM_CHANGECBCHAIN = 0x030D,
        WM_HSCROLLCLIPBOARD = 0x030E,
        WM_QUERYNEWPALETTE = 0x030F,
        WM_PALETTEISCHANGING = 0x0310,
        WM_PALETTECHANGED = 0x0311,
        WM_HOTKEY = 0x0312,
        WM_PRINT = 0x0317,
        WM_PRINTCLIENT = 0x0318,
        WM_THEME_CHANGED = 0x031A,
        WM_HANDHELDFIRST = 0x0358,
        WM_HANDHELDLAST = 0x035F,
        WM_AFXFIRST = 0x0360,
        WM_AFXLAST = 0x037F,
        WM_PENWINFIRST = 0x0380,
        WM_PENWINLAST = 0x038F,
        WM_APP = 0x8000,
        WM_USER = 0x0400,
        WM_REFLECT = WM_USER + 0x1c00
    }
    internal enum ControlsId : int
    {
        ButtonOk = 0x1,
        ButtonCancel = 0x2,
        ButtonHelp = 0x40E,//0x0000040e
        GroupFolder = 0x440,
        LabelFileType = 0x441,
        LabelFileName = 0x442,
        LabelLookIn = 0x443,
        DefaultView = 0x461,
        LeftToolBar = 0x4A0,
        ComboFileName = 0x47c,
        ComboFileType = 0x470,
        ComboFolder = 0x471,
        CheckBoxReadOnly = 0x410
    }
    #endregion
    public static class NativeImports
    {
        [DllImport("User32.Dll")]
        internal static extern int GetDlgCtrlID(IntPtr hWndCtl);
    }

    public class EventWizardWindowClosed : EventArgs
    {
        public int CloseReason;
    }

    public delegate void WizardWindowClosedDelegate(object sender, EventWizardWindowClosed evclosed);

    public interface IWizardWindow : IDisposable, IServiceProvider
    {
        DialogResult ShowWizardWindow(IWin32Window parent);
        bool IsShown { get; }
        IWin32Window VSWindow { get; }
        event WizardWindowClosedDelegate WizardWindowClosed;

    }
    public interface ILanguageProvider : IServiceProvider
    {
        string[] Extensions { get; }
        bool IsValidExtension(string ext);
        bool CanAddToProject(ITemplatePattern ipt);
        IEnumerable<FileData> GenerateCodeFiles(ITemplatePattern ipt);
        IEnumerable<ITemplatePattern> LoadTemplates(ITabWizard itw, string providerName);
        string PatternsVersion { get; }
        string TemplatesFolder { get; }
    }
    public static class ProjectUtils
    {
        public static bool IsIncludedProvider(string name) { return name.StartsWith("Default",StringComparison.InvariantCultureIgnoreCase); }
        public static T GetAssemblyAttribute<T>(this System.Reflection.Assembly ass) 
            where T : Attribute
        {
            object[] attributes = ass.GetCustomAttributes(typeof(T), false);
            if (attributes == null || attributes.Length == 0)
                return null;
            return attributes.OfType<T>().SingleOrDefault();
        }
        public static T Object2T<T>(object o)
        {
            return (T)o;
        }
        public static T[] GetEnumValues<T>() where T : struct, IComparable, IFormattable, IConvertible
        {
            if (typeof(T).BaseType != typeof(Enum))
            {
                throw new ArgumentException(string.Format("{0} is not of type System.Enum", typeof(T)));
            }
            return Enum.GetValues(typeof(T)) as T[];
        }

   

        static internal DateTime LastOnlineCheck
        {
            get
            {
                RegistryKey reg = null;
                try
                {
                    reg = Registry.CurrentUser.OpenSubKey(BaseLanguageProvider.AppHive, false);
                    if (reg != null)
                    {

                        long ticks = (long)reg.GetValue("LastOnlineCheck", 0L);
                        return (ticks == 0L) ? DateTime.Today : new DateTime(ticks);
                    }
                    else
                    {
                        return DateTime.Today;
                    }
                }
                catch (Exception)
                {

                    return DateTime.Today;
                }
                finally
                {
                    if (reg != null)
                        reg.Close();
                }
            }
            set
            {
                using (RegistryKey reg = Registry.CurrentUser.CreateSubKey(BaseLanguageProvider.AppHive, RegistryKeyPermissionCheck.ReadWriteSubTree))
                {
                    reg.SetValue("LastOnlineCheck", (ulong)value.Ticks, RegistryValueKind.QWord);
                }
            }
        }
        static internal string LastOnlineVer
        {
            get
            {
                RegistryKey reg = null;
                try
                {
                    reg = Registry.CurrentUser.OpenSubKey(BaseLanguageProvider.AppHive, false);
                    if (reg != null)
                    {
                        return reg.GetValue("LastOnlineVersion", "9.9") as string;
                    }
                    else
                    {
                        return "9.9";
                    }
                }
                catch (Exception)
                {

                    return "9.9";
                }
                finally
                {
                    if (reg != null)
                        reg.Close();
                }
            }
            set
            {
                using (RegistryKey reg = Registry.CurrentUser.CreateSubKey(BaseLanguageProvider.AppHive, RegistryKeyPermissionCheck.ReadWriteSubTree))
                {
                    reg.SetValue("LastOnlineVersion", value, RegistryValueKind.String);
                }
            }
        }
        internal const string BaseURL = "http://vsdesignpatterns.codeplex.com/";
#if VS2008
        public const string VSVersion = "9.0";
        public const string VSYear = "2008";
#endif
#if VS2010
            public const string VSVersion = "10.0";
            public const string VSYear = "2010";
#endif
#if VS2012
            public const string VSVersion = "11.0";
            public const string VSYear = "2012";
#endif
#if VS2013
            public const string VSVersion = "12.0";
            public const string VSYear = "2013";
#endif
        internal static bool NewOnlineVersionExists(out string downloadUrl, out float onlineVersion)
        {

            bool isnew = false;
            downloadUrl = null;
            onlineVersion = 0;

            downloadUrl = BaseURL;
            try
            {

                using (WebClient client = new WebClient())
                {
                    string result = client.DownloadString(downloadUrl);
                    string version = result.FindFirstMatchingGroup(@"<td>\s*VS[ ]*Design[ ]*Patterns[ ]*(?<Version>[0-9,.]+)\s*</td>", "Version", RegexOptions.IgnoreCase).Trim();

                    if (!string.IsNullOrEmpty(version))
                    {

                        ProjectUtils.LastOnlineVer = version;
                        onlineVersion = float.Parse(version, System.Globalization.NumberStyles.Float);
                        isnew = IsNewerThanCurrent(onlineVersion);
                        if (isnew)
                        {
                            string latestUrl = result.FindFirstMatchingGroup("href=\\\"(?<URL>" + BaseURL + "releases/view/[0-9]+)\\\"", "URL", RegexOptions.Multiline);
                            //string downloadUrl2 = result.FindFirstMatchingGroup(@"href=" +'"' + "(?<URL>" + baseurl + "releases/view/[0-9]+)" +'"', "URL", RegexOptions.Multiline);
                            if (string.IsNullOrEmpty(latestUrl))
                            {
                                return true;
                            }
                            else
                                downloadUrl = latestUrl;
                            result = client.DownloadString(downloadUrl);
                            //get specific download link
                            latestUrl = result.FindFirstMatchingGroup("href=\\\"(?<URL>" + BaseURL + "downloads/get/[0-9]+)\\\"" + ".+>DesignPatterns" + VSYear + "_" + version + ".zip\\s*</a>", "URL", RegexOptions.Multiline);
                            //latestUrl = result.FindFirstMatchingGroup("href=\\\"(?<URL>" + BaseURL + "downloads/get/[0-9]+)\\\"" + ".+>DesignPatterns2008_" + version + ".zip\\s*</a>", "URL", RegexOptions.Multiline);
                            if (latestUrl != null && latestUrl.Length > 20)//pass the smell test
                            {
                                downloadUrl = latestUrl;//best url to download
                                return true;
                            }
                            //get download link for all setups if can't find specific version
                            latestUrl = result.FindFirstMatchingGroup("href=\\\"(?<URL>" + BaseURL + "downloads/get/[0-9]+)\\\"" + ".+>DesignPatternsAllSetups_" + version + ".zip\\s*</a>", "URL", RegexOptions.Multiline);
                            if (string.IsNullOrEmpty(latestUrl))//no urll for download all, get the first download link on the page
                                latestUrl = result.FindFirstMatchingGroup("href=\\\"(?<URL>" + BaseURL + "downloads/get/[0-9]+)\\\"", "URL", RegexOptions.Multiline);
                            if (string.IsNullOrEmpty(latestUrl) || latestUrl.Length <= 20)//can't pass the smell test
                            {
                                return true;//keep the last valid url
                            }
                            downloadUrl = latestUrl;
                        }
                    }

                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(Application.OpenForms[0], ex.Message, "Exception in IsnewVersion");
            }
            return isnew;
        }

        internal static bool IsNewerThanCurrent(float onlineVersion)
        {
            Assembly crt = Assembly.GetExecutingAssembly();

            Version v = crt.GetName().Version;
            float crtVersion = float.Parse(string.Format("{0}.{1}", v.Major, v.Minor), System.Globalization.NumberStyles.Number);
            return crtVersion < onlineVersion;
        }
        internal static string FindFirstMatchingGroup(this string input, string regex, string group, RegexOptions opt)
        {
            string result = null;
            try
            {
                Match m = Regex.Match(input, regex, opt);
                if (m.Success && m.Groups[group].Success)
                {
                    result = m.Groups[group].Value;
                }
            }
            catch
            {

                result = null;
            }
            return result;
        }
        static public T GetAsService<T>(this IServiceProvider sp) where T : class
        {
            return sp.GetService(typeof(T)) as T;
        }
        static public string Replace(this string str, string oldValue, string newValue, StringComparison comparison)
        {
            StringBuilder sb = new StringBuilder(str.Length);

            try
            {
                int previousIndex = 0;
                int index = str.IndexOf(oldValue, comparison);
                while (index != -1)
                {
                    sb.Append(str.Substring(previousIndex, index - previousIndex));
                    sb.Append(newValue);
                    index += oldValue.Length;

                    previousIndex = index;
                    index = str.IndexOf(oldValue, index, comparison);
                }
                sb.Append(str.Substring(previousIndex));

                return sb.ToString();
            }
            finally
            {
                sb.Length = 0;
            }
        }
        static public void RemoveReadOnlyAttribute(this System.IO.DirectoryInfo dir, System.IO.SearchOption so)
        {
            try
            {
                // Take a snapshot of the file system.
                // This method assumes that the application has discovery permissions 
                // for all folders under the specified path.
                IEnumerable<System.IO.FileInfo> fileList = dir.GetFiles("*.*", so);//System.IO.SearchOption.AllDirectories);

                foreach (FileInfo f in fileList)
                {
                    if ((uint)(f.Attributes & FileAttributes.ReadOnly) != 0)
                    {
                        f.Attributes ^= FileAttributes.ReadOnly;// remove read only
                    }
                }

            }
            catch //(Exception ex)
            {

                throw;
            }

        }

        /*
        [StructLayout(LayoutKind.Sequential)]
        internal struct RECT
        {
            internal int Left;
            internal int Top;
            internal int Right;
            internal int Bottom;
            public override string ToString()
            {
                return string.Format("Left = {0}, Top = {1}, Right = {2}, Bottom ={3}",
                    Left, Top, Right, Bottom);
            }
            internal int Width
            {
                get { return Math.Abs(Right - Left); }
            }
            internal int Height
            {
                get { return Math.Abs(Bottom - Top); }
            }
        }
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool IsWindow(IntPtr hWnd);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool GetWindowRect(IntPtr hWnd, out RECT lpRect);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool IsWindowVisible(IntPtr hWnd);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool IntersectRect(out RECT lprcDst, [In] ref RECT lprcSrc1,
           [In] ref RECT lprcSrc2);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool IsRectEmpty([In] ref RECT lprc);
        [DllImport("user32.dll", SetLastError = true)]
        static extern IntPtr FindWindowEx(IntPtr hwndParent, IntPtr hwndChildAfter, string lpszClass, string lpszWindow);

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        internal static extern int GetWindowTextLength(IntPtr hWnd);
        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        internal static extern int GetWindowText(IntPtr hWnd, StringBuilder lpString, int nMaxCount);
        internal delegate bool EnumThreadDelegate(IntPtr hWnd, IntPtr lParam);
        [DllImport("User32.Dll")]
        internal static extern void GetClassName(IntPtr hWnd, System.Text.StringBuilder param, int length);

        [DllImport("user32.dll", SetLastError = false)]
        internal static extern IntPtr GetDesktopWindow();
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool EnumThreadWindows(uint dwThreadId, EnumThreadDelegate lpfn, IntPtr lParam);
        static RECT _DesktopRect;
        */

        //see http://stackoverflow.com/questions/2454456/cancel-installation-from-my-custom-action
        static public DialogResult ShowModalDlg(string message, string title, MessageBoxButtons buttons, MessageBoxIcon icon)
        {//for the msi exec the window is with caption 'DesignPatterns for VS 2010' and class MsiDialogCloseClass, or 'Windows Installer' and '#32770 (Dialog)'
            // Create a host form that is a TopMost window which will be the 
            // parent of the MessageBox.


            NativeWindow nativeWnd = new NativeWindow();
            try
            {
                IntPtr hWnd = (from p in Process.GetProcessesByName("msiexec")
                               where (p.MainWindowHandle != IntPtr.Zero && (p.MainWindowTitle == "DesignPatterns for VS " + ProjectUtils.VSYear || p.MainWindowTitle == "Windows Installer"))
                               select p.MainWindowHandle).SingleOrDefault();
                //nativeWnd = NativeWindow.FromHandle(hWnd); //does not work

                //IntPtr hwnd = Process.GetCurrentProcess().MainWindowHandle; is always 0
                if (hWnd == IntPtr.Zero)
                    return MessageBox.Show(message, title, buttons, icon, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly);
                else
                {
                    nativeWnd.AssignHandle(hWnd);
                    return MessageBox.Show(nativeWnd, message, title, buttons, icon);
                }

            }
            catch (Exception ex)
            {
                return MessageBox.Show(ex.Message, "Exception thrown in ShowModalDlg", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly);
            }
            finally { nativeWnd.ReleaseHandle(); }

        }
        /*
               class WindowWrapper : IWin32Window
               {
                   [DllImport("user32.dll")]
                   [return: MarshalAs(UnmanagedType.Bool)]
                   static extern internal bool IsWindowVisible(IntPtr hWnd);
                   internal WindowWrapper(IntPtr handle)
                   {
                       Handle = handle;
                   }

                   public IntPtr Handle { get; private set; }
               }
       
               static public IWin32Window GetMainWindow(this System.Diagnostics.Process pr)
               {
                   if (pr == null || pr.MainWindowHandle == IntPtr.Zero || !WindowWrapper.IsWindowVisible(pr.MainWindowHandle))
                       return null;
                   return new WindowWrapper(pr.MainWindowHandle);
               }
                */
    }
    public enum ResultEnum
    {
        Success = 0, UnknownError = 1, ItemExists = 2, FileExists = 3, AutomationError = 4, MissingFileTemplate = 5, BadFileExtension = 6
    }
    public class EventValidaton : EventArgs
    {
        public int SelIndex;
        public IEnumerable<String> Files;
        public string ErrorMsg;
        public ResultEnum Result;
        public ILanguageProvider LangProvider;
    }
    public class EventGeneration : EventArgs
    {
        public FileData[] Files;
        public string ErrorMsg;
        public ResultEnum Result;
        public ILanguageProvider LangProvider;
    }
    public delegate void CanUpdateVSProjectDelegate(IServiceProvider sender, EventValidaton evval);
    public delegate void UpdateVSProjectDelegate(IServiceProvider sender, EventGeneration evgen);
    [Flags]
    public enum CodeLangEnum : uint
    {
        [Description("Unknown Project Type")]
        Unsupported = 0, VB = 1, CSharp = 2, NativeCpp = 4, ManagedCpp = 8, JSharp = 16, FSharp = 32
    }
    [Flags]
    public enum LocationsEnum : uint
    {
        None = 0,
        ContextProject = 1,
        Item = 2,
        Folder = 4,
        Tools = 8,
        Standard = 16,
        MenuProject = 32
    }
    [Flags]
    public enum ColumnsEnum : ulong
    {
        PatternAlias = 1,
        MinVSVersion = 2,
        CompilerSyntax = 4,
        ProgrammingStyle = 8,
        Category = 16,
        Culture = 32
    }
    public interface IProjectInfo : IServiceProvider
    {
        string ProjectName { get; }
        string FileName { get; }
        CodeLangEnum CodeLang { get; }
        IEnumerable<ILanguageProvider> CreateSpecificLanguageProviders(Predicate<string> filter);
        TimeSpan LoadingTime { get; }
        IWizardWindow WizardWindow { get; }
        Dictionary<string, string> ProjectProperties { get; }
        event CanUpdateVSProjectDelegate CanUpdateVSProject;
        event UpdateVSProjectDelegate UpdateVSProject;
        void OnValidatePattern(object sender, EventValidaton evval);
        void OnUpdateVSProject(object sender, EventGeneration evgen);
    }

    public class FileData
    {
        public string itemName;
        public string fileNameCopy;
    }
}
