﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DesignPatternsWizard
{
    using System.Windows.Forms;
    using System.Xml.Linq;
    using System.IO;
    using Microsoft.Win32;
    using Ionic.Zip;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.ComponentModel.Composition.Hosting;
    using DesignPatternsWizard;

    public enum License
    {
        Free = 0,
        Trial = 1,
        Premium = 2,
    }
    public interface ILanguageProviderMetaData
    {//see below for custom attribute
        [DefaultValue(CodeLangEnum.CSharp)]
        CodeLangEnum Language { get; }
        [DefaultValue("")]
        string Description { get; }
        [DefaultValue("")]
        string LanguageProviderName { get; }
        [DefaultValue(null)]
        CodeLangEnum[] AdditionalLanguages { get; }
        [DefaultValue(null)]
        string[] Extensions { get; }
        [DefaultValue(false)]
        License LicenseType { get; }

    }

    [MetadataAttribute]

    [AttributeUsage(AttributeTargets.Class, AllowMultiple = false)]

    public class ExportLanguageProviderAttribute : ExportAttribute, ILanguageProviderMetaData
    {
        public CodeLangEnum Language { get; set; }
        public string Description { get; set; }
        public string LanguageProviderName { get; set; }
        public CodeLangEnum[] AdditionalLanguages { get; set; }
        public string[] Extensions { get; set; }
        public License LicenseType { get; set; }
        public ExportLanguageProviderAttribute()
            : base(typeof(ILanguageProvider))
        {
            LicenseType = License.Free;
            Description = LanguageProviderName = string.Empty;
            AdditionalLanguages = null;
        }

    }

    public abstract class BaseLanguageProvider : IServiceProvider
    {
        public const string AppHive = @"Software\DesignPatterns\VS" + ProjectUtils.VSYear;
        public abstract string TemplatesFolder { get; protected set; }
        public virtual string[] Extensions
        {
            get
            {
                return this.GetAsService<ILanguageProviderMetaData>().Extensions;
            }
        }
        public bool IsValidExtension(string ext)
        {
            if (ext == ".*")
                return true;
            string lext = ext.ToLower();
            return Extensions.Any(s => s == lext);
        }

        protected abstract string Logo { get; set; }
        public IEnumerable<FileData> GenerateCodeFiles(ITemplatePattern ipt)
        {
            return ipt.TemplateFiles.Select(t =>
            {
                if (t == null || string.IsNullOrEmpty(t.TemplateContent))
                    return null;
                string tmpPath = Path.GetTempFileName();
                File.AppendAllText(tmpPath, Logo + System.Environment.NewLine + ExecuteArgSubstitution(t, ipt.Arguments), System.Text.Encoding.UTF8);
                return new FileData() { fileNameCopy = tmpPath, itemName = t.FileItemName };

            });
        }
        public static void UnzipLibrary(string baseDir, string libPath)
        {
            try
            {
                //clear the old template dir first
                if (Directory.Exists(libPath))
                {
                    DirectoryInfo di = new DirectoryInfo(libPath);
                    di.RemoveReadOnlyAttribute(System.IO.SearchOption.AllDirectories);
                    di.Delete(true);
                }
                // Specifying Console.Out here causes diagnostic msgs to be sent to the Console
                // In a WinForms or WPF or Web app, you could specify nothing, or an alternate
                // TextWriter to capture diagnostic messages.

                var options = new ReadOptions { StatusMessageWriter = System.Console.Out };
                using (ZipFile zip = ZipFile.Read(Path.Combine(baseDir, "TemplateLibrary.zip"), options))
                {
                    // This call to ExtractAll() assumes:
                    //   - none of the entries are password-protected.
                    //   - want to extract all entries to current working directory
                    //   - none of the files in the zip already exist in the directory;
                    //     if they do, the method will throw.
                    zip.ExtractAll(baseDir);
                }
            }
            catch (Exception)
            {
                // ProjectUtils.ShowModal(ex.Message, "Exception thrown while unzipping",MessageBoxButtons.OK,MessageBoxIcon.Error);
                throw;
            }
        }
        public static LocationsEnum GetLocations()
        {
            LocationsEnum loc = (LocationsEnum)15;
            RegistryKey reg = Registry.CurrentUser.OpenSubKey(BaseLanguageProvider.AppHive, false);
            if (reg != null)
                loc = (LocationsEnum)reg.GetValue("Locations", loc, RegistryValueOptions.None);
            return loc;
        }
        public const LocationsEnum DefaultLocations = (LocationsEnum)39;
        public static LocationsEnum Locations
        {
            get
            {
                LocationsEnum loc = DefaultLocations;
                RegistryKey reg = Registry.CurrentUser.OpenSubKey(BaseLanguageProvider.AppHive, false);
                if (reg != null)
                {
                    object obj = reg.GetValue("Locations", loc, RegistryValueOptions.None);
                    loc = (LocationsEnum)System.Convert.ToUInt32(obj);
                    reg.Close();
                }
                return loc;
            }
            set
            {
                RegistryKey reg = Registry.CurrentUser.CreateSubKey(BaseLanguageProvider.AppHive, RegistryKeyPermissionCheck.Default);
                if (reg != null)
                {
                    reg.SetValue("Locations", value, RegistryValueKind.DWord);
                    reg.Close();
                }

            }
        }
        public const ColumnsEnum DefaultColumns = (ColumnsEnum)31;//all but culture
        public static ColumnsEnum Columns
        {
            get
            {
                ColumnsEnum columns = BaseLanguageProvider.DefaultColumns;
                RegistryKey reg = Registry.CurrentUser.OpenSubKey(BaseLanguageProvider.AppHive, false);
                if (reg != null)
                {
                    object obj = reg.GetValue("Columns", columns, RegistryValueOptions.None);
                    columns = (ColumnsEnum)System.Convert.ToUInt64(obj);
                    reg.Close();
                }
                return columns;
            }
            set
            {
                RegistryKey reg = Registry.CurrentUser.CreateSubKey(BaseLanguageProvider.AppHive, RegistryKeyPermissionCheck.Default);
                if (reg != null)
                {
                    reg.SetValue("Columns", value, RegistryValueKind.QWord);
                    reg.Close();
                }

            }
        }
        public static ProjectInfo _projectInfo;
        public string PatternsVersion { get; protected set; }
        protected string PatternDefinitions { get; set; }
        static internal readonly bool _Is64bitProc;

        static BaseLanguageProvider()
        {

            try
            {
                _Is64bitProc = System.Runtime.InteropServices.Marshal.SizeOf(typeof(IntPtr)) == 8;
            }
            catch (Exception)
            {

                // throw;
            }
        }

        public BaseLanguageProvider()
        {
            try
            {

                RegistryKey reg = Registry.CurrentUser.OpenSubKey(AppHive, false);
                if (reg != null)
                {
                    TemplatesFolder = reg.GetValue("TemplateLibPath", null, RegistryValueOptions.None) as string;
                    reg.Close();
                }
                if (string.IsNullOrEmpty(TemplatesFolder))
                {//not installed
                    TemplatesFolder = GetDevelopmentPatternsDir();
                }
            }
            catch
            {
            }

        }
        private string GetDevelopmentPatternsDir()
        {
            string crtdir = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
            DirectoryInfo di = Directory.GetParent(crtdir);

            string[] args = Environment.GetCommandLineArgs();
            string executable = Path.GetFileName(args[0]);
            if (executable.ToLower() == "devenv.exe")
                return Path.Combine(di.Parent.FullName, @"Misc\TemplateLibrary");
            else
#if VS2013
                return Path.Combine(di.Parent.Parent.FullName, @"Misc\TemplateLibrary");//fix the vs2013 deviation
#else
                return Path.Combine(di.Parent.Parent.Parent.FullName, @"Misc\TemplateLibrary");
#endif


        }
        protected virtual bool IsLanguageCompatible(string lng)
        {
            return string.Compare(lng, _projectInfo.CodeLang.ToString(), true) == 0;
        }
        public IEnumerable<ITemplatePattern> LoadTemplates(ITabWizard itw, string providerName)
        {
            IWizardWindow iwnd = _projectInfo.GetAsService<IWizardWindow>();
            try
            {
                if (!File.Exists(PatternDefinitions))
                {
                    string err = string.Format("Can not find the file '{0}'!\r\nRestarting Visual Studio might fix it.", PatternDefinitions);
                    if (iwnd != null && iwnd.VSWindow != null)
                        MessageBox.Show(iwnd.VSWindow, err, "Can not load templates", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    else
                        MessageBox.Show(err, "Can not load templates", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return null;
                }
                XDocument doc = XDocument.Load(PatternDefinitions, LoadOptions.None);
                PatternsVersion = (doc.Root.Attribute("PatternsVersion") == null) ? null : doc.Root.Attribute("PatternsVersion").Value;
                //selects only relevant nodes based on language
                IEnumerable<XElement> templs = doc.Root.Elements("PatternTemplate").Where(x => IsLanguageCompatible(x.Attribute("CodeLang").Value));//.Where(x => x.Attribute("CodeLang").Value == this._projectInfo.CodeLang.ToString());
                return templs.Select(e => new TemplatePattern(this as ILanguageProvider, e) as ITemplatePattern);
            }
            catch (Exception ex)
            {
                if (iwnd != null && iwnd.VSWindow != null)
                    MessageBox.Show(iwnd.VSWindow, ex.Message, "Can not load templates");
                else
                    MessageBox.Show(ex.Message, "Can not load templates");
                return null;
            }

        }

        static internal string ExecuteArgSubstitution(ITemplateFile itf, IEnumerable<ITemplateArgument> args)
        {
            try
            {
                string result = itf.TemplateContent;
                foreach (ITemplateArgument arg in args)
                {
                    string target = string.Format("${0}$", arg.ArgumentName);
                    result = result.Replace(target, arg.Value, StringComparison.InvariantCultureIgnoreCase);
                }

                return result;
            }
            catch
            {

                throw;
            }
        }

        #region IServiceProvider Members

        public virtual object GetService(Type serviceType)
        {
            if (serviceType == typeof(ILanguageProvider) || serviceType == typeof(IServiceProvider))
                return this;
            object res = null;
            if (serviceType == typeof(IProjectInfo))
                res = (_projectInfo == null) ? null : _projectInfo.GetService(serviceType);
            if (res != null)
                return res;
            return res;
        }

        #endregion
    }
    #region CSharpProvider

    [ExportLanguageProvider(LanguageProviderName = "Default CSharp", Description = "CSharp LangProvider",
    Language = CodeLangEnum.CSharp, LicenseType = License.Free, Extensions = new string[] { ".cs" },
    AdditionalLanguages = new CodeLangEnum[] { CodeLangEnum.CSharp })]
    public class DefaultCSharpProvider : BaseLanguageProvider, ILanguageProvider
    {

        private DefaultCSharpProvider()//(IProjectInfo projectInfo)
        {
            Logo = @"//This source code was generated using Visual Studio Design Patterns add-in
//You can find the source code and binaries at http://VSDesignPatterns.codeplex.com";
            PatternDefinitions = System.IO.Path.Combine(TemplatesFolder, "CSharpTemplates.xml");
        }

        #region IServiceProvider Members

        public override object GetService(Type serviceType)
        {
            if (_projectInfo != null && serviceType == typeof(ILanguageProviderMetaData))
                return _projectInfo["Default CSharp"];
            return base.GetService(serviceType);
        }

        #endregion
        #region ILanguageProvider Members
        protected override string Logo { get; set; }


        public override string TemplatesFolder { get; protected set; }

        public bool CanAddToProject(ITemplatePattern ipt)
        {
            return true;
        }

        #endregion
    }
    #endregion

    #region VBProvider
    [ExportLanguageProvider(LanguageProviderName = "Default VB", Description = "VB LangProvider",
    Language = CodeLangEnum.VB, LicenseType = License.Free, Extensions = new string[] { ".vb" },
    AdditionalLanguages = new CodeLangEnum[] { CodeLangEnum.VB })]
    public class DefaultVBProvider : BaseLanguageProvider, ILanguageProvider
    {

        private DefaultVBProvider()
        {
            Logo = @"'This source code was generated using Visual Studio Design Patterns add-in
'You can find the source code and binaries at http://VSDesignPatterns.codeplex.com";
            PatternDefinitions = System.IO.Path.Combine(TemplatesFolder, "VBTemplates.xml");

        }
        protected override string Logo { get; set; }

        #region IServiceProvider Members

        public override object GetService(Type serviceType)
        {
            if (_projectInfo != null && serviceType == typeof(ILanguageProviderMetaData))
                return _projectInfo["Default VB"];
            return base.GetService(serviceType);
        }

        #endregion

        #region ILanguageProvider Members
        public override string TemplatesFolder { get; protected set; }


        public bool CanAddToProject(ITemplatePattern ipt)
        {
            return true;
        }

        #endregion
    }
    #endregion

    #region CPPProvider
    [ExportLanguageProvider(LanguageProviderName = "Default CPP", Description = "CPP LangProvider",
    Language = CodeLangEnum.ManagedCpp | CodeLangEnum.NativeCpp, LicenseType = License.Free,
    Extensions = new string[] { ".h", ".cpp", ".hpp", ".c", ".hxx", ".tli", ".tlh" },
    AdditionalLanguages = new CodeLangEnum[] { CodeLangEnum.NativeCpp, CodeLangEnum.ManagedCpp })]
    public class DefaultCPPProvider : BaseLanguageProvider, ILanguageProvider
    {

        private DefaultCPPProvider()
        {
            Logo = @"//This source code was generated using Visual Studio Design Patterns add-in
//You can find the source code and binaries at http://VSDesignPatterns.codeplex.com";
            PatternDefinitions = System.IO.Path.Combine(TemplatesFolder, "CPPTemplates.xml");

        }
        #region IServiceProvider Members

        public override object GetService(Type serviceType)
        {
            if (_projectInfo != null && serviceType == typeof(ILanguageProviderMetaData))
                return _projectInfo["Default CPP"];
            return base.GetService(serviceType);
        }

        #endregion
        #region ILanguageProvider Members
        protected override string Logo { get; set; }
        protected override bool IsLanguageCompatible(string lng)
        {
            switch (lng.ToLower())
            {
                case "nativecpp":
                    return true;//always good for managed + unmanaged
                case "managedcpp":
                    return _projectInfo.CodeLang == CodeLangEnum.ManagedCpp;//managed code good only for managed projects
                default:
                    return false;

            }
            //return string.Compare(lng, _projectInfo.CodeLang.ToString(), true) == 0;
        }
        public override string TemplatesFolder { get; protected set; }


        public bool CanAddToProject(ITemplatePattern ipt)
        {
            return true;
        }

        #endregion
    }
    #endregion
}
