﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace DesignPatternsWizard
{
    using System.IO;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.ComponentModel.Composition;
    using System.ComponentModel.Composition.Hosting;
    using LanguageProviderEx = Lazy<ILanguageProvider, ILanguageProviderMetaData>;

    public class ProjectInfo : IProjectInfo
    {
        readonly Stopwatch _stopWatch = new Stopwatch();
        public ProjectInfo(CodeLangEnum pt)
        {
            _stopWatch.Start();
            this.CodeLang = pt;    
        }


        static readonly Dictionary<CodeLangEnum, IEnumerable<ILanguageProvider>> _langProvidersDict = new Dictionary<CodeLangEnum, IEnumerable<ILanguageProvider>>(3);//3 languages supported

        #region IServiceProvider Members

        public object GetService(Type serviceType)
        {
            if (serviceType == typeof(IProjectInfo) || serviceType == typeof(IServiceProvider))
                return this;
            object res = null;
            if (serviceType == typeof(IEnumerable<ILanguageProvider>))
            {
                IEnumerable<ILanguageProvider> prv = _langProvidersDict[this.CodeLang];
                return prv;
            }
             return res;

        }

        #endregion
        #region IProjectInfo Members

        public event CanUpdateVSProjectDelegate CanUpdateVSProject;
        public event UpdateVSProjectDelegate UpdateVSProject;
        public void OnUpdateVSProject(object sender, EventGeneration evgen)
        {

            if (UpdateVSProject == null || UpdateVSProject.GetInvocationList().Length < 1)
            {
                evgen.ErrorMsg = string.Format("No handler found for GeneratePattern event");
                evgen.Result = ResultEnum.UnknownError;
                return;
            }
            evgen.Result = ResultEnum.Success;

            UpdateVSProject(sender as IServiceProvider, evgen);


        }
        public void OnValidatePattern(object sender, EventValidaton evval)
        {

            evval.ErrorMsg = null;
            evval.Result = ResultEnum.Success;
            switch (evval.SelIndex)
            {
                case 2:

                    if (CanUpdateVSProject == null || CanUpdateVSProject.GetInvocationList().Length < 1)
                    {
                        evval.ErrorMsg = string.Format("No handler found for CanGeneratePattern event");
                        evval.Result = ResultEnum.UnknownError;
                        return;
                    }
                    CanUpdateVSProject(sender as IServiceProvider, evval);
                    break;
                default:
                    evval.Result = ResultEnum.Success;
                    evval.ErrorMsg = null;
                    break;
            }
        }
        public string ProjectName { get; set; }

        public string FileName { get; set; }

        public CodeLangEnum CodeLang { get; set; }

        public IWizardWindow WizardWindow { get; internal set; }
        public TimeSpan LoadingTime
        {
            get
            {
                _stopWatch.Stop();
                return _stopWatch.Elapsed;
            }
        }

        public ILanguageProviderMetaData this[string lpName]
        {
            get
            {
                try
                {
                    LanguageProviderEx elemEx = LangProviders.SingleOrDefault(x => string.Compare(x.Metadata.LanguageProviderName,lpName,true)==0);
                    if (elemEx == null)
                        return null;
                    return elemEx.Metadata;
                }
                catch (Exception)
                {
                    return null;
                }
            }
        }

        static bool IsLanguageSupported(LanguageProviderEx lpex, CodeLangEnum lang)
        {
            return (lpex.Metadata.Language & lang) != default(CodeLangEnum);
            //return lpex.Metadata.AdditionalLanguages.Any(l => l == lang);
        }
        public IEnumerable<ILanguageProvider> this[CodeLangEnum lang]
        {

            get
            {
                return LangProviders.Where(x => IsLanguageSupported(x, lang)).Select(e => e.Value);
            }

        }

        #region MEF
        [ImportMany(typeof(ILanguageProvider))]
        public IEnumerable<LanguageProviderEx> LangProviders{  get;  set; }
        
        public void InitMEF()
        {
            try
            {
                //An aggregate catalog that combines multiple catalogs
                var catalog = new AggregateCatalog();
                //Adds all the parts found in all assemblies in the same directory as the executing program  
                //catalog.Catalogs.Add(new DirectoryCatalog(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location)));
                catalog.Catalogs.Add(new AssemblyCatalog(Assembly.GetExecutingAssembly()));
                //Create the CompositionContainer with the parts in the catalog
                CompositionContainer container = new CompositionContainer(catalog);
                //Fill the imports of this object
                container.ComposeParts(this);
                //var langlookupEx = LangProviders.GroupBy(x => x.Metadata.Language);
                //var langlookup = LangProviders.GroupBy(x => x.Metadata.Language, y => y.Value);
            }

            catch (CompositionException compositionException)
            {
                Trace.TraceError("CompositionException thrown in  InitMEF:{0}", compositionException);
            }
            catch (System.Reflection.ReflectionTypeLoadException ex)
            {
                Trace.TraceError("ReflectionTypeLoadException thrown in  InitMEF:{0}", ex);
            }
            catch (Exception ex)
            {
                Trace.TraceError("Exception thrown in  InitMEF:{0}", ex);
            }
        }
        #endregion

        public IEnumerable<ILanguageProvider> CreateSpecificLanguageProviders(Predicate<string> filter)
        {
            BaseLanguageProvider._projectInfo = this;
            try
            {
                IEnumerable<ILanguageProvider> langProviders = null;
                if (_langProvidersDict.TryGetValue(this.CodeLang, out langProviders) && langProviders != null)
                    return langProviders;
                InitMEF();
              langProviders = (filter==null)?this[CodeLang]: LangProviders.Where(x => IsLanguageSupported(x, CodeLang) && filter(x.Metadata.LanguageProviderName)).Select(e => e.Value);
                 // langProviders = LangProviders.Where(x => IsLanguageSupported(x, CodeLang) && filter(x.Metadata.LanguageProviderName)).Select(e => e.Value);              
                if (langProviders != null)
                    _langProvidersDict.Add(CodeLang, langProviders);
                return langProviders;
            }
            catch (Exception ex)
            {
                Trace.TraceError("Exception thrown in  CreateSpecificLanguageProviders:{0}", ex);
                return null;
            }
        }
        #endregion

        public Dictionary<string, string> ProjectProperties { get; set; }
    }
}
