﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DesignPatternsWizard
{
    using System.Xml.Linq;
    using System.IO;
    using System.Globalization;


    public class TemplateArgument : ITemplateArgument
    {
        ITemplatePattern _parentTemplate;
        public TemplateArgument(ITemplatePattern parentTemplate, XElement xmlArgument)
        {
            _parentTemplate = parentTemplate;
            if (xmlArgument == null || xmlArgument.Name != "TemplateArgument")
                return;
            if (xmlArgument.Attribute("ArgumentName") != null)
                ArgumentName = xmlArgument.Attribute("ArgumentName").Value;
            if (xmlArgument.Attribute("Description") != null)
                Description = xmlArgument.Attribute("Description").Value;
            Value = xmlArgument.Value;

        }
        #region IServiceProvider Members

        public object GetService(Type serviceType)
        {
            if (serviceType == typeof(ITemplateArgument) || serviceType == typeof(IServiceProvider))
                return this;
            object res = null;
            if (serviceType == typeof(ITemplatePattern))
                res = (_parentTemplate == null) ? null : _parentTemplate.GetService(serviceType);
            return res;
        }

        #endregion
        #region ITemplateArgument Members
        public string ArgumentName { get; private set; }
        public string Description { get; private set; }
        public string Value { get; set; }
        public bool IsRuntime { get; private set; }
        #endregion


    }
    public class TemplateRuntimeArgument : ITemplateArgument
    {
        ITemplatePattern _parentTemplate;
        IProjectInfo _projectInfo;
        public TemplateRuntimeArgument(ITemplatePattern parentTemplate, IProjectInfo projectInfo, XElement xmlArgument)
        {
            _parentTemplate = parentTemplate;
            _projectInfo = projectInfo;
            if (xmlArgument == null || xmlArgument.Name != "TemplateArgument")
                return;
            if (xmlArgument.Attribute("ArgumentName") != null)
                ArgumentName = xmlArgument.Attribute("ArgumentName").Value;
            if (xmlArgument.Attribute("Description") != null)
                Description = xmlArgument.Attribute("Description").Value;
            string val;
            if (projectInfo.ProjectProperties.TryGetValue(ArgumentName, out val))
                Value = val;
        }
        #region IServiceProvider Members

        public object GetService(Type serviceType)
        {
            if (serviceType == typeof(TemplateRuntimeArgument) || serviceType == typeof(IServiceProvider))
                return this;
            object res = null;
            if (serviceType == typeof(IProjectInfo))
                res = (_projectInfo == null) ? null : _projectInfo.GetService(serviceType);
            if (res != null)
                return res;
            res = (_parentTemplate == null) ? null : _parentTemplate.GetService(serviceType);
            return res;
        }

        #endregion
        #region ITemplateArgument Members
        public string ArgumentName { get; private set; }
        public string Description { get; private set; }
        public string Value { get; set; }
        public bool IsRuntime { get; private set; }
        #endregion


    }
    public interface ITemplateFile : IServiceProvider
    {
        string FileItemName { get; }
        string FileTemplate { get; }
        string StoredAs { get; }
        string TemplateContent { get; }

    }
    public class TemplateFile : ITemplateFile
    {

        TemplatePattern _parentTemplate;

        FileData GeneratePatternFile(IProjectInfo projectInfo, TemplateFile templateFile)
        {
            return null;
        }
        public TemplateFile(TemplatePattern parentTemplate, XElement fileElement)
        {
            _parentTemplate = parentTemplate;
            if (fileElement == null || fileElement.Name != "FileTemplate")
                return;
            FileItemName = fileElement.Attribute("FileItemName").Value;
            FileTemplate = fileElement.Value;
            if (fileElement.Attribute("StoredAs") != null)
                StoredAs = fileElement.Attribute("StoredAs").Value;
            try
            {

                if (StoredAs == "File")
                {
                    string path = GetTemplateLangFolder();
                    if (!File.Exists(path))
                        return;
                    TextReader tr = File.OpenText(path);
                    TemplateContent = tr.ReadToEnd();
                }
            }
            catch //(Exception ex)
            {
            }
        }

        private string GetTemplateLangFolder()
        {

            if (Path.IsPathRooted(FileTemplate))
                return FileTemplate;
            ILanguageProvider lp = _parentTemplate.GetAsService<ILanguageProvider>();
            IProjectInfo pi = _parentTemplate.GetAsService<IProjectInfo>();
            CodeLangEnum lang = pi.CodeLang;
            string codeFolder = null;
            switch (lang)
            {
                case CodeLangEnum.ManagedCpp:
                case CodeLangEnum.NativeCpp:
                    codeFolder = "Cpp";
                    break;
                default:
                    codeFolder = lang.ToString();
                    break;


            }
            return Path.Combine(lp.TemplatesFolder, string.Format("{0}\\{1}\\{2}",
                _parentTemplate.PatternName, codeFolder, FileTemplate));

        }
        public string FileItemName { get; set; }
        public string FileTemplate { get; set; }
        public string StoredAs { get; private set; }
        public string TemplateContent { get; private set; }
        #region IServiceProvider Members

        public object GetService(Type serviceType)
        {
            if (serviceType == typeof(ITemplateFile) || serviceType == typeof(IServiceProvider))
                return this;
            object res = null;
            res = (_parentTemplate == null) ? null : _parentTemplate.GetService(serviceType);
            return res;
        }



        #endregion
    }
    public interface ITemplatePattern : IServiceProvider
    {
        string PatternName { get; set; }
        string PatternAlias { get; set; }
        string Description { get; set; }
        string DescriptionStoredAs { get; set; }
        float MinVSVersion { get; set; }// will filter features like optional params, async/await,dynamic
        string Category { get; set; }//behavioral,creational

        string CompilerSyntax { get; set; }//managed,native,VC10, empty
        string ProgrammingStyle { get; set; }//generic,template,plain,nothing
        string Culture { get; set; }
        ITabWizard ParentTabWizard { get; set; }
        IEnumerable<ITemplateArgument> Arguments { get; set; }
        CodeLangEnum CodeLanguage { get; set; }//string version of CodeLanguageEnum
        object Picture { get; set; }
        string PictureStoredAs { get; set; }
        string OnlineResources { get; set; }
        IEnumerable<ITemplateFile> TemplateFiles { get; set; }
    }


    public class TemplatePattern : IServiceProvider, ITemplatePattern
    {
        readonly static string _currentCulture = System.Windows.Forms.InputLanguage.CurrentInputLanguage.Culture.ThreeLetterISOLanguageName;
        readonly static string[] _countryCodes;
        public string ConvertThreeLetterNameToTwoLetterName(string name)
        {
            if (name.Length != 3)
            {
                throw new ArgumentException("name must be three letters.");
            }

            name = name.ToUpper();

            CultureInfo[] cultures = CultureInfo.GetCultures(CultureTypes.SpecificCultures);
            foreach (CultureInfo culture in cultures)
            {
                RegionInfo region = new RegionInfo(culture.LCID);
                if (region.ThreeLetterISORegionName.ToUpper() == name)
                {
                    return region.TwoLetterISORegionName;
                }
            }

            return null;
        }
        static TemplatePattern()
        {
            //see http://stackoverflow.com/questions/4884692/converting-country-codes-in-net
            CultureInfo[] cultures = CultureInfo.GetCultures(CultureTypes.SpecificCultures);
            _countryCodes = cultures.Select(culture =>
                {
                    try
                    {
                        var region = new RegionInfo(culture.LCID);
                        return region.ThreeLetterISORegionName;
                    }
                    catch (Exception)
                    {
                        //var consoleColor = Console.ForegroundColor;

                        //Console.ForegroundColor = ConsoleColor.Red;
                        //Console.WriteLine("Culture not found: " + culture.Name);
                        //Console.ForegroundColor = consoleColor;
                        return null;
                    }
                }).Where(s => s != null).ToArray();

        }
        ILanguageProvider _languageProvider;
        readonly string[] _descrExts;
        readonly string[] _pictExts;
        public TemplatePattern(ILanguageProvider languageProvider, XElement templateElem)
        {
            _languageProvider = languageProvider;
            if (templateElem == null || templateElem.Attribute("PatternName") == null
                || templateElem.Attribute("CodeLang") == null || templateElem.Name != "PatternTemplate")
            {
                return;//bad xml
            }
            PatternName = templateElem.Attribute("PatternName").Value;
            try
            {
                this.CodeLanguage = (CodeLangEnum)Enum.Parse(typeof(CodeLangEnum), templateElem.Attribute("CodeLang").Value, true);
            }
            catch
            {

                CodeLanguage = CodeLangEnum.Unsupported;
            }
            var properties = templateElem.Element("PatternProperties");
            PictureStoredAs = "File";
            DescriptionStoredAs = "Text";
            Culture = "Neutral";
            _descrExts = new string[] { ".htm", ".html", ".rtf", ".txt" };
            _pictExts = new string[] { ".png", ".jpg", ".gif", ".bmp" };
            if (properties != null)
            {
                float minVSVer = 9.0F;
                foreach (XElement e in properties.Elements())
                {
                    switch (e.Name.ToString())//.ToLower()
                    {
                        case "PatternAlias":
                            PatternAlias = e.Value;
                            break;
                        case "MinVSVersion":
                            if (!float.TryParse(e.Value, out minVSVer))
                                minVSVer = 9.0F;
                            MinVSVersion = minVSVer;
                            break;
                        case "Picture":
                            if (e.Attribute("StoredAs") != null)
                                PictureStoredAs = e.Attribute("StoredAs").Value;
                            Picture = e.Value;
                            if (string.Compare(PictureStoredAs, "File", true) == 0)
                            {
                                if (!Path.IsPathRooted(Picture as string))
                                {

                                    Picture = Path.Combine(_languageProvider.TemplatesFolder, string.Format("{0}\\{1}",
                                         PatternName, Picture));
                                }
                            }
                            break;

                        case "OnlineResources":
                            OnlineResources = e.Value;
                            break;
                        case "CompilerSyntax":
                            CompilerSyntax = e.Value;
                            break;

                        case "ProgrammingStyle":
                            ProgrammingStyle = e.Value;
                            break;
                        case "Description":
                            if (e.Attribute("StoredAs") != null)
                                DescriptionStoredAs = e.Attribute("StoredAs").Value;
                            Description = e.Value;
                            break;

                        case "Category":
                            Category = e.Value;
                            break;
                        case "Culture":
                            Culture = e.Value;//"Neutral" if missing
                            break;

                        default:
                            break;



                    }
                }
            }
            string defaultPath = Path.Combine(_languageProvider.TemplatesFolder, string.Format("{0}\\{1}", PatternName, PatternName));
            if (Description == null)
            {//try adding default file if value not in xml file

                //check for local language
                if (Description == null)
                {
                    Description = _descrExts.Select(e => string.Format("{0}_{1}{2}",defaultPath,_currentCulture,e))
                        .FirstOrDefault(s => File.Exists(s));
                }
                //check for Neutral language
                if (Description == null)
                {
                    Description = _descrExts.Select(e => defaultPath + e).FirstOrDefault(s => File.Exists(s));
                }
                if (Description != null)
                    DescriptionStoredAs = "File";
            }
            if (Picture == null)
            {//try adding default file if value not in xml file
                Picture = _pictExts.Select(e => defaultPath + e).FirstOrDefault(s => File.Exists(s));
                if (Picture != null)
                    PictureStoredAs = "File";
            }
            IProjectInfo projectInfo = _languageProvider.GetAsService<IProjectInfo>();
            if (OnlineResources == null)
            {

                OnlineResources = string.Format(@"www.bing.com/search?q={0}+software+design+pattern+{1}", PatternName, projectInfo.CodeLang);
            }
            var files = templateElem.Element("FileTemplates");
            if (files != null)
            {
                //iterate it
                TemplateFiles = files.Elements("FileTemplate").Select(e => new TemplateFile(this, e) as ITemplateFile).ToArray();
            }
            var xmlargs = templateElem.Element("TemplateArguments");
            if (xmlargs != null)
            {
                Arguments = xmlargs.Elements("TemplateArgument").Select(e =>
                    {
                        if (e.Attribute("IsRuntime") == null || e.Attribute("IsRuntime").Value.ToLower() == "false")
                        {
                            return new TemplateArgument(this, e) as ITemplateArgument;
                        }
                        else
                        {
                            return new TemplateRuntimeArgument(this, projectInfo, e) as ITemplateArgument;
                        }
                    });
            }
        }

        public IEnumerable<ITemplateArgument> Arguments { get; set; }
        public float MinVSVersion { get; set; }// will filter features like optional params, async/await,dynamic
        public object Picture { get; set; }
        public string PictureStoredAs { get; set; }
        public string OnlineResources { get; set; }
        public string CompilerSyntax { get; set; }//managed,native,VC10, empty
        public string ProgrammingStyle { get; set; }//generic,template,plain,nothing
        public string PatternName { get; set; }
        public string PatternAlias { get; set; }
        public string Description { get; set; }
        public string DescriptionStoredAs { get; set; }
        public CodeLangEnum CodeLanguage { get; set; }//string version of CodeLanguageEnum
        public string Category { get; set; }//behavioral,creational
        public string Culture { get; set; }
        public IEnumerable<ITemplateFile> TemplateFiles { get; set; }



        public ITabWizard ParentTabWizard { get; set; }
        #region IServiceProvider Members

        public object GetService(Type serviceType)
        {
            if (serviceType == typeof(ITemplatePattern) || serviceType == typeof(IServiceProvider))
                return this;
            if (serviceType == typeof(ILanguageProvider))
                return _languageProvider;
            object res = null;
            res = (_languageProvider == null) ? null : _languageProvider.GetService(serviceType);
            return res;
        }

        #endregion
    }
}
