﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;
using System.Reflection;

namespace DesignPatternsWizard
{
    using System.Collections.Generic;
    using System.IO;
    using System.Threading;
#if !VS2008
    using System.Threading.Tasks;
#endif

    public partial class WizardForm : Form, IWizardWindow
    {
        internal IEnumerable<ILanguageProvider> _langProviders;
        internal readonly IProjectInfo _projectInfo;
        bool _bUserCancelling = false;

        private WizardForm()
        {
            InitializeComponent();
        }
        public WizardForm(IProjectInfo projectInfo)
            : this()
        {
            _projectInfo = projectInfo;
            if (_projectInfo != null)
            {
                (_projectInfo as ProjectInfo).WizardWindow = this;
            }
        }
        bool _bShowNewVersionAvailable;
        
        private void WizardForm_Load(object sender, EventArgs e)
        {
            if (DesignMode)
                return;

            try
            {

                _wizardPages.SelectedIndex = 0;
                _buttonPrev.Enabled = false;
                this.UseWaitCursor = true;
                foreach (Control c in this.Controls)
                    c.Enabled = false;
                if (_projectInfo != null)
                {
#if VS2008
                    BackgroundWorker bw = new BackgroundWorker();
                    bw.DoWork += new DoWorkEventHandler((object snd, DoWorkEventArgs resultTask) =>
                    {
                        _langProviders = _projectInfo.CreateSpecificLanguageProviders(ProjectUtils.IsIncludedProvider);
                        CheckVersion();
                        IEnumerable<ITemplatePattern> templates = LoadPatterns(_langProviders);// lps.First().LoadTemplates(_wizardPages, "default");
                        resultTask.Result = (templates == null || templates.Count() == 0) ? null : new SortableBindingList<ITemplatePattern>(templates.ToList());
                    });
                    bw.RunWorkerCompleted += new RunWorkerCompletedEventHandler((object snd, RunWorkerCompletedEventArgs resultTask) =>

#else
                    Task<SortableBindingList<ITemplatePattern>> tsk = new Task <SortableBindingList<ITemplatePattern>>(() =>
                        {
                            _langProviders = _projectInfo.CreateSpecificLanguageProviders(ProjectUtils.IsIncludedProvider);
                        CheckVersion();
                            IEnumerable<ITemplatePattern> templates = LoadPatterns(_langProviders);

                            return (templates == null || templates.Count() == 0) ? null : new SortableBindingList<ITemplatePattern>(templates.ToList());
                        });
                    tsk.ContinueWith(resultTask =>
#endif

                    {
                        try
                        {
                            this.UseWaitCursor = false;
                            foreach (Control c in this.Controls)
                                c.Enabled = true;

                            if (_langProviders == null || _langProviders.Count() == 0)
                            {
                                MessageBox.Show(this, string.Format("the {0} does not have a coresponding LanguageProvider for {1}", _projectInfo.ProjectName, _projectInfo.CodeLang));
                                return;
                            }
#if VS2008
                            if (resultTask.Error != null)
                            {
                                _toolStripStatusLabel.Text = "Unable to load Design Templates because " + resultTask.Error.Message;
                                return;
                            }
#endif

                            Text += string.Format(" - Project: {0}, Lang:{1}, Assembly Version:{2}", _projectInfo.ProjectName, _projectInfo.CodeLang, Assembly.GetExecutingAssembly().GetName().Version);//_langProviders.First().PatternsVersion);
#if DEBUG
                            Text += ":Debug";
#endif
                            if (resultTask.Result != null)
                            {
                                _wizardPages.Templates = resultTask.Result as SortableBindingList<ITemplatePattern>;
                                _dataGridViewPatterns.DataSource = _wizardPages.Templates;
                                _toolStripStatusLabel.Text = string.Format("Loaded {0} {1} software patterns from '{2}' folder in {3} ms",
                                    _wizardPages.Templates.Count(), _projectInfo.CodeLang, _langProviders.First().TemplatesFolder, System.Convert.ToUInt32(_projectInfo.LoadingTime.TotalMilliseconds));
                                _lblCrtVSVer.Text = string.Format("The current VS version is {0}", ProjectUtils.VSVersion);//ipi.VisualStudioVersion);
                                SetFilters();
                                ShowColumns(); //invoked by FilterComboBox_SelectedValueChanged too
                            }
                            _bformLoaded = true;
                            if (_bShowNewVersionAvailable)
                            {

                                AskForUpgrade(this);
                            }

                        }
                        catch (Exception ex)
                        {
                            _toolStripStatusLabel.Text = string.Format("Exception thrown while loading:{0} from '{1}' folder", ex.Message, _langProviders.First().TemplatesFolder);
                        }
#if VS2008
                    });
                    bw.RunWorkerAsync();

#else
                    }, TaskScheduler.FromCurrentSynchronizationContext());
                    tsk.Start();
#endif
                }//end if  if (ipi != null && lp != null)
                this.CenterToParent();
                _labelLang.Text += InputLanguage.CurrentInputLanguage.Culture.ThreeLetterISOLanguageName;
                //_comboBoxCategory.Focus();//won't work
            }
            catch (Exception ex)
            {
                _toolStripStatusLabel.Text = string.Format("Exception thrown while loading:{0} from '{1}' folder", ex.Message, _langProviders.First().TemplatesFolder);
            }

        }

        internal static void AskForUpgrade(IWin32Window parent)
        {
            if (DialogResult.Yes == MessageBox.Show(parent, string.Format("Version {0} has become available{1}Do you want to download it now?",
                ProjectUtils.LastOnlineVer, Environment.NewLine),
                "An update is available online", MessageBoxButtons.YesNo, MessageBoxIcon.Question))
            {
                string downloadUrl;
                float onlineVersion;
                //remove the time component to signal that the user saw the upgrade Msg
                ProjectUtils.LastOnlineCheck = DateTime.Today;
                if (ProjectUtils.NewOnlineVersionExists(out downloadUrl, out onlineVersion))
                {
                    System.Diagnostics.Process.Start(string.IsNullOrEmpty(downloadUrl) ? ProjectUtils.BaseURL : downloadUrl);
                }
            }
            else
                ProjectUtils.LastOnlineCheck = ProjectUtils.LastOnlineCheck.Date;
        }
        IEnumerable<ITemplatePattern> LoadPatterns(IEnumerable<ILanguageProvider> lps)
        {
            List<ITemplatePattern> lst = new List<ITemplatePattern>();
            foreach (ILanguageProvider lp in lps)
            {
                lst.AddRange(lp.LoadTemplates(_wizardPages, "default"));
            }
            return lst;
        }
        private void CheckVersion()
        {
            DateTime lastCheck = ProjectUtils.LastOnlineCheck;
            bool bFirstTime = ProjectUtils.LastOnlineVer == "9.9" && lastCheck < DateTime.Now;
            if (bFirstTime)
            {
                Version v = Assembly.GetExecutingAssembly().GetName().Version;
                string crtVersion = string.Format("{0}.{1}", v.Major, v.Minor);

                ProjectUtils.LastOnlineVer = crtVersion;
                //remove the time component to signal that the user saw the upgrade Msg
                ProjectUtils.LastOnlineCheck = DateTime.Today;

                _bShowNewVersionAvailable = false;
            }
            else
            {//not the first time
                _bShowNewVersionAvailable = ProjectUtils.IsNewerThanCurrent(float.Parse(ProjectUtils.LastOnlineVer, System.Globalization.NumberStyles.Float));
                _bShowNewVersionAvailable &= lastCheck != lastCheck.Date && lastCheck < DateTime.Now;
            }
        }

        private void SetFilters()
        {
            //select the first row
            List<string> cbxData = new List<string>(new string[] { "Any" });
            cbxData.AddRange(_wizardPages.Templates.Where(p => p.CompilerSyntax != null).Select(t => t.CompilerSyntax).Distinct());
            _comboBoxCompilerSyntax.DataSource = cbxData.ToArray();//make a copy since cbxData is reused
            AssociateFilters2Columns(_comboBoxCompilerSyntax, ColumnsEnum.CompilerSyntax);


            cbxData.Clear();
            cbxData.Add("Any");
            cbxData.AddRange(_wizardPages.Templates.Where(p => p.ProgrammingStyle != null).Select(t => t.ProgrammingStyle).Distinct());
            _comboBoxProgrammingStyle.DataSource = cbxData.ToArray();
            AssociateFilters2Columns(_comboBoxProgrammingStyle, ColumnsEnum.ProgrammingStyle);


            cbxData.Clear();
            cbxData.Add("Any");
            cbxData.AddRange(_wizardPages.Templates.Where(p => p.MinVSVersion != 0.0f).Select(t => t.MinVSVersion.ToString()).Distinct());
            _comboBoxMinVSVersion.DataSource = cbxData.ToArray();
            AssociateFilters2Columns(_comboBoxMinVSVersion, ColumnsEnum.MinVSVersion);

            cbxData.Clear();
            cbxData.Add("Any");
            cbxData.AddRange(_wizardPages.Templates.Where(p => p.Category != null).Select(t => t.Category).Distinct());
            _comboBoxCategory.DataSource = cbxData.ToArray();
            AssociateFilters2Columns(_comboBoxCategory, ColumnsEnum.Category);

            cbxData.Clear();
            cbxData.Add("Any");
            cbxData.AddRange(_wizardPages.Templates.Where(p => p.Culture != null).Select(t => t.Culture).Distinct());
            _comboBoxCulture.DataSource = cbxData.ToArray();
            AssociateFilters2Columns(_comboBoxCulture, ColumnsEnum.Culture);
        }

        private void AssociateFilters2Columns(ComboBox cbx, ColumnsEnum colEnum)
        {
            DataGridViewColumn column = _dataGridViewPatterns.Columns[colEnum.ToString()];
            column.Tag = cbx;//make associations
            cbx.Tag = column;
        }


        private void ShowColumns()
        {
            _dataGridViewPatterns.Columns["Arguments"].Visible = false;
            _dataGridViewPatterns.Columns["TemplateFiles"].Visible = false;
            _dataGridViewPatterns.Columns["ParentTabWizard"].Visible = false;
            _dataGridViewPatterns.Columns["Picture"].Visible = false;
            _dataGridViewPatterns.Columns["Description"].Visible = false;
            _dataGridViewPatterns.Columns["CodeLanguage"].Visible = false;
            _dataGridViewPatterns.Columns["OnlineResources"].Visible = false;
            _dataGridViewPatterns.Columns["PictureStoredAs"].Visible = false;
            _dataGridViewPatterns.Columns["DescriptionStoredAs"].Visible = false;
            ColumnsEnum cols = BaseLanguageProvider.Columns;
            ColumnsEnum[] allColumns = ProjectUtils.GetEnumValues<ColumnsEnum>();
            foreach (ColumnsEnum col in allColumns)
            {

                switch (col)
                {
                    case ColumnsEnum.Culture:
                        SetColumn(cols, col, _comboBoxCulture, _dataGridViewPatterns);
                        break;
                    case ColumnsEnum.Category:
                        SetColumn(cols, col, _comboBoxCategory, _dataGridViewPatterns);
                        break;
                    case ColumnsEnum.ProgrammingStyle:
                        SetColumn(cols, col, _comboBoxProgrammingStyle, _dataGridViewPatterns);
                        break;
                    case ColumnsEnum.CompilerSyntax:
                        SetColumn(cols, col, _comboBoxCompilerSyntax, _dataGridViewPatterns);
                        break;
                    case ColumnsEnum.MinVSVersion:
                        SetColumn(cols, col, _comboBoxMinVSVersion, _dataGridViewPatterns);
                        break;
                    default://this is the pattern alias that can't be filtered
                        _dataGridViewPatterns.Columns[col.ToString()].Visible = (cols & col) != default(ColumnsEnum);
                        break;

                }
                //this might be redundant most of the times
                _dataGridViewPatterns.Columns[col.ToString()].DataPropertyName = col.ToString();

            }
            Application.DoEvents();

        }

        private static void SetColumn(ColumnsEnum cols, ColumnsEnum col, ComboBox cbx, DataGridView dgr)
        {
            bool unfiltered = false;
            string selVal = cbx.SelectedValue == null ? null : cbx.SelectedValue.ToString();
            unfiltered = (string.IsNullOrEmpty(selVal) || selVal == "Any");
            cbx.Enabled = (cols & col) != default(ColumnsEnum);
            if (!cbx.Enabled)// avoid extra filtering
                cbx.SelectedItem = "Any";
            DataGridViewColumn column = dgr.Columns[col.ToString()];
            column.Visible = unfiltered && cbx.Enabled;
        }


        private void _wizardPages_SelectedIndexChanged(object sender, EventArgs e)
        {
            int index = _wizardPages.SelectedIndex;
            switch (index)
            {
                case 0:
                    _buttonNext.Text = "&Next";
                    _buttonPrev.Enabled = false;
                    break;
                case 1:
                    _buttonNext.Text = "&Next";
                    _buttonPrev.Enabled = true;
                    //populate using _wizardPages.SelectedPattern
                    LoadTemplateArguments(_wizardPages.SelectedPattern);

                    break;
                case 2:
                    int i = 0;
                    _tabControlFiles.SuspendLayout();
                    _tabControlFiles.Controls.Clear();
                    foreach (ITemplateFile itf in _wizardPages.SelectedPattern.TemplateFiles)
                    {
                        _tabControlFiles.Controls.Add(new FilePage(itf, i++));
                    }
                    _tabControlFiles.ResumeLayout(true);
                    this.PerformLayout();
                    if (i > 0)
                    {
                        _buttonNext.Text = "&Generate";
                        _buttonPrev.Enabled = true;
                    }
                    else
                        _toolStripStatusLabel.Text = string.Format("The Template '{0}' has no files", _wizardPages.SelectedPattern.PatternName);
                    break;
                default:
                    return;
            }

            IPageWizard ipage = _wizardPages.SelectedTab as IPageWizard;
            if (this._langProviders.FirstOrDefault() != null && ipage != null)
            {
                ITabWizard itw = this._wizardPages as ITabWizard;
                ILanguageProvider ilp = itw.SelectedPattern.GetAsService<ILanguageProvider>();
                EventValidaton evval = ipage.Validate(index, ilp);
                _buttonNext.Enabled = evval.Result == ResultEnum.Success;
                //_buttonNext.Enabled = true;//!!!!!remove this test only
                if (evval.ErrorMsg != null && !_buttonNext.Enabled)
                    _toolStripStatusLabel.Text = evval.ErrorMsg;
                else
#if DEBUG

                    _toolStripStatusLabel.Text = string.Format(" _SelectedIndex = {0}, Name:{1}", index, sender.ToString());
#else
                    _toolStripStatusLabel.Text = string.Empty;
#endif

            }
        }

        private void LoadTemplateArguments(ITemplatePattern itp)
        {
            //avoid updates if the parent template is the same
            if (itp != _dataGridViewCustomParams.Tag)
            {
                _dataGridViewCustomParams.Tag = itp;
                itp.Arguments = itp.Arguments.ToArray();//enumerate args
                _dataGridViewCustomParams.DataSource = itp.Arguments.OfType<TemplateArgument>().ToArray();
                _dataGridViewCustomParams.Columns["IsRuntime"].Visible = false;
                _dataGridViewCustomParams.Columns["ArgumentName"].ReadOnly = true;
                _dataGridViewCustomParams.Columns["Description"].ReadOnly = true;
            }
            if (_dataGridViewRuntimeParams.Tag != itp)
            {
                _dataGridViewRuntimeParams.Tag = itp;
                _dataGridViewRuntimeParams.DataSource = itp.Arguments.OfType<TemplateRuntimeArgument>().ToArray();
                _dataGridViewRuntimeParams.Columns["IsRuntime"].Visible = false;
                _dataGridViewRuntimeParams.Columns["ArgumentName"].ReadOnly = true;
                _dataGridViewRuntimeParams.Columns["Description"].ReadOnly = true;
            }
        }

        private void _buttonNext_Click(object sender, EventArgs e)
        {
            if (_wizardPages.SelectedIndex == 2)
            {
                //IProjectInfo ipi = this.GetAsService<IProjectInfo>();
                float VSversion = float.Parse(ProjectUtils.VSVersion);//ipi.VisualStudioVersion);
                if (VSversion < _wizardPages.SelectedPattern.MinVSVersion &&
                    MessageBox.Show(this,
                    "The VS version is lower than the pattern's VS version.\r\nDo you still want to generate the code?",
                    "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No)
                {
                    _wizardPages.SelectedIndex = 0;
                }
                else
                    Close();
            }
            else
                _wizardPages.SelectedIndex += 1;
        }

        private void _buttonCancel_Click(object sender, EventArgs e)
        {
            _bUserCancelling = true;
            Close();
        }

        private void _buttonPrev_Click(object sender, EventArgs e)
        {
            if (_wizardPages.SelectedIndex == 0)
                return;
            _wizardPages.SelectedIndex -= 1;
        }


        private void WizardForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            switch (e.CloseReason)
            {
                case CloseReason.UserClosing:
                    if (!_bUserCancelling)
                    {
                        var lp = this.GetAsService<ILanguageProvider>();
                        EventGeneration evg = new EventGeneration() { LangProvider = lp, Result = ResultEnum.UnknownError };
                        evg.Files = lp.GenerateCodeFiles(_wizardPages.SelectedPattern).Where(n => n != null).ToArray();
                        if (evg.Files == null || evg.Files.Length < 1)
                        {
                            evg.Result = ResultEnum.UnknownError;
                            evg.ErrorMsg = "Can't generate from bad template";
                            _toolStripStatusLabel.Text = evg.ErrorMsg;
                            e.Cancel = true;
                            return;
                        }
                        IProjectInfo ipi = lp.GetAsService<IProjectInfo>();
                        ipi.OnUpdateVSProject(this, evg);
                        e.Cancel = evg.Result != ResultEnum.Success;
                        if (evg.ErrorMsg != null && e.Cancel)
                        {
                            MessageBox.Show(this, evg.ErrorMsg, "DesigPatterWizard", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            _toolStripStatusLabel.Text = evg.ErrorMsg;
                        }
                        else
                            _toolStripStatusLabel.Text = "";
                    }
                    break;
                case CloseReason.ApplicationExitCall:
                    break;
                case CloseReason.None:
                    e.Cancel = false;
                    break;
                default:
                    break;
            }
        }

        private void WizardForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (WizardWindowClosed != null)
                WizardWindowClosed(sender, new EventWizardWindowClosed() { CloseReason = (int)e.CloseReason });
            if (ProjectUtils.LastOnlineCheck + new TimeSpan(15, 0, 0, 0) < DateTime.Today)// < is correct
            {//check only if older than 15 days

                Thread tr = new Thread(new ThreadStart(() =>
                {
                    string downloadUrl;
                    float onlineVersion;
                    //keep the time component to signal that the user haven't seen the upgrade Msg
                    ProjectUtils.LastOnlineCheck = DateTime.Now;
                    ProjectUtils.NewOnlineVersionExists(out downloadUrl, out onlineVersion);
                }));
                tr.IsBackground = false;
                tr.Start();
            }
        }
        #region Overrides

        protected override void WndProc(ref Message m)
        {

            switch ((Msg)m.Msg)
            {
                case Msg.WM_CREATE:
                    break;

                case Msg.WM_ACTIVATE:

                    break;
                case Msg.WM_SYSCOMMAND:
                    if (m.HWnd == this.Handle && (int)m.WParam == (int)SystemCommands.SC_CLOSE)
                        _bUserCancelling = true;
                    break;
                case Msg.WM_CLOSE:
                    break;
                case Msg.WM_DESTROY:
                    break;
                case Msg.WM_NCDESTROY:

                case Msg.WM_QUIT:
                    break;
                case Msg.WM_COMMAND:
                    switch (NativeImports.GetDlgCtrlID(m.LParam))
                    {
                        case (int)ControlsId.ButtonOk://OK
                            break;
                        case (int)ControlsId.ButtonCancel://Cancel
                            break;
                        case (int)ControlsId.ButtonHelp://help
                            break;
                        case 0:
                            break;
                        default:
                            break;
                    }
                    break;
                default:
                    break;
            }//switch ((Msg)m.Msg) ends
            base.WndProc(ref m);
        }
        #endregion

        #region IProjectInfo Members

        public string FileName { get; set; }

        public string Language { get; set; }
        public string VisualStudioVersion { get; set; }

        public Dictionary<string, string> ProjectProperties { get; set; }

        #endregion

        #region IWizardWindow Members

        public DialogResult ShowWizardWindow(IWin32Window parent)
        {
            this.Show(parent);
            return DialogResult.OK;
        }

        public bool IsShown
        {
            get { return this.IsShown; }

        }
        public IWin32Window VSWindow { get; set; }


        public event WizardWindowClosedDelegate WizardWindowClosed;

        #endregion

        #region IServiceProvider Members

        public new object GetService(Type serviceType)
        {
            if (serviceType == typeof(IWizardWindow) || serviceType == typeof(IServiceProvider))
                return this;
            if (serviceType == typeof(IEnumerable<ILanguageProvider>) && _langProviders != null)
                return _langProviders;

            if (serviceType == typeof(IProjectInfo) && _projectInfo != null)
                return _projectInfo;
            object res = null;
            if (res != null)
                return res;
            if (serviceType == typeof(ILanguageProvider))
            {
                if (_bformLoaded)
                {
                    ITabWizard itw = this._wizardPages as ITabWizard;
                    if (itw.SelectedPattern == null)
                        return null;
                    ILanguageProvider ilp = itw.SelectedPattern.GetAsService<ILanguageProvider>();
                    res = ilp;
                }
                else
                    res = (_langProviders == null || _langProviders.FirstOrDefault() == null) ? null : _langProviders.FirstOrDefault().GetService(serviceType);
            }
            if (res != null)
                return res;
            if (serviceType == typeof(ITabWizard))
                return _wizardPages;
            if (res == null)
                res = base.GetService(serviceType);
            return res;
        }

        #endregion
        bool _bformLoaded = false;
        private void FilterComboBox_SelectedValueChanged(object sender, EventArgs e)
        {
            ComboBox cb = sender as ComboBox;
            if (cb == null)
                return;
            string selVal = cb.SelectedValue as string;

            if (_bformLoaded)
            {
                if (cb.Tag != null)
                {
                    ColumnsEnum cols = BaseLanguageProvider.Columns;
                    DataGridViewColumn dgvc = cb.Tag as DataGridViewColumn;
                    string columnName = dgvc.Name;
                    ColumnsEnum col = (ColumnsEnum)Enum.Parse(typeof(ColumnsEnum), columnName, true);
                    SetColumn(cols, col, cb, _dataGridViewPatterns);
                }
                else
                {
                }
                // ShowColumns(); old style
            }

            IEnumerable<ITemplatePattern> filteredTemplates = _wizardPages.Templates;//must be already enumerated (array)
            //filter the templates
            selVal = _comboBoxMinVSVersion.SelectedValue as string;
            if (!string.IsNullOrEmpty(selVal) && selVal != "Any")
                filteredTemplates = filteredTemplates.Where(t => t.MinVSVersion >= Convert.ToSingle(selVal)).ToArray();

            selVal = _comboBoxCompilerSyntax.SelectedValue as string;
            if (!string.IsNullOrEmpty(selVal) && selVal != "Any")
                filteredTemplates = filteredTemplates.Where(t => string.Compare(t.CompilerSyntax, selVal, true) == 0).ToArray();

            selVal = _comboBoxProgrammingStyle.SelectedValue as string;
            if (!string.IsNullOrEmpty(selVal) && selVal != "Any")
                filteredTemplates = filteredTemplates.Where(t => string.Compare(t.ProgrammingStyle, selVal, true) == 0).ToArray();

            selVal = _comboBoxCategory.SelectedValue as string;
            if (!string.IsNullOrEmpty(selVal) && selVal != "Any")
                filteredTemplates = filteredTemplates.Where(t => string.Compare(t.Category, selVal, true) == 0).ToArray();

            selVal = _comboBoxCulture.SelectedValue as string;
            if (!string.IsNullOrEmpty(selVal) && selVal != "Any")
                filteredTemplates = filteredTemplates.Where(t => string.Compare(t.Culture, selVal, true) == 0).ToArray();

            _dataGridViewPatterns.DataSource = new SortableBindingList<ITemplatePattern>(filteredTemplates);
        }

        private void _linkLabelOnlineResources_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            p.StartInfo.FileName = _linkLabelOnlineResources.Links[0].LinkData.ToString();
            p.Start();
        }

        private void _dataGridViewPatterns_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                DataGridView dgv = sender as DataGridView;
                IList<ITemplatePattern> patterns = dgv.DataSource as IList<ITemplatePattern>;
                if (patterns == null || e.RowIndex < 0)
                {
                    _wizardPages.SelectedPattern = null;
                    return;
                }
                _buttonNext.Enabled = true;

                _wizardPages.SelectedPattern = dgv.Rows[e.RowIndex].DataBoundItem as ITemplatePattern; //patterns[e.RowIndex];
                _richTextBoxDescription.Clear();
                switch (_wizardPages.SelectedPattern.DescriptionStoredAs.ToLower())
                {
                    case "text":
                        string txt = _wizardPages.SelectedPattern.Description.Replace(@"\r\n", Environment.NewLine).Replace(@"\n", Environment.NewLine);
                        _webBrowserDescription.Visible = false;
                        _richTextBoxDescription.Visible = true;
                        _richTextBoxDescription.Text = txt;
                        break;
                    case "file":
                        string path = _wizardPages.SelectedPattern.Description;
                        if (!Path.IsPathRooted(path))
                        {
                            path = Path.Combine(_langProviders.FirstOrDefault().TemplatesFolder, string.Format("{0}\\{1}",
                                _wizardPages.SelectedPattern.PatternName, path));
                        }
                        switch (Path.GetExtension(path).ToLower())
                        {
                            case ".txt":
                                _webBrowserDescription.Visible = false;
                                _richTextBoxDescription.Visible = true;
                                _richTextBoxDescription.LoadFile(path, RichTextBoxStreamType.PlainText);
                                break;
                            case ".html":
                            case ".htm":
                                _richTextBoxDescription.Visible = false;
                                _webBrowserDescription.Visible = true;
                                _webBrowserDescription.Navigate(path,false);//must change to html control
                                break;
                            default:
                                _richTextBoxDescription.Visible = true;
                                _webBrowserDescription.Visible = false;
                                _richTextBoxDescription.LoadFile(path);
                                break;

                        }
                        break;
                    default:
                        break;
                }

                _linkLabelOnlineResources.Links.Clear();
                string str = "OnlineResources:" + _wizardPages.SelectedPattern.OnlineResources.Substring(0, Math.Min(50, _wizardPages.SelectedPattern.OnlineResources.Length));
                _linkLabelOnlineResources.Text = str;
                _linkLabelOnlineResources.Links.Add("OnlineResources:".Length, str.Length,
                    "\"" + _wizardPages.SelectedPattern.OnlineResources + "\"");
                if (string.Compare(_wizardPages.SelectedPattern.PictureStoredAs, "File", true) == 0)
                    _pictureBox.ImageLocation = _wizardPages.SelectedPattern.Picture as string;
                else
                    throw new NotImplementedException(_wizardPages.SelectedPattern.PictureStoredAs + " format is not implemented for pictures");


            }
            catch (Exception ex)
            {
                _toolStripStatusLabel.Text = ex.Message;

            }

        }


        private void _dataGridViewPatterns_DataSourceChanged(object sender, EventArgs e)
        {
            DataGridView dgv = sender as DataGridView;
            IList<ITemplatePattern> patterns = dgv.DataSource as IList<ITemplatePattern>;
            if (patterns == null || patterns.Count == 0)
            {
                _buttonNext.Enabled = false;
                _richTextBoxDescription.Clear();
                _linkLabelOnlineResources.Links.Clear();
                _linkLabelOnlineResources.Text = "";
                Application.DoEvents();
                if (_pictureBox.Image != null)
                    _pictureBox.Image.Dispose();
                _pictureBox.Image = null;
                Application.DoEvents();
                if (_pictureBox.InitialImage != null)
                    _pictureBox.InitialImage.Dispose();
                _pictureBox.InitialImage = null;
                _pictureBox.Update();
                Application.DoEvents();
                _pictureBox.Refresh();
                _wizardPages.SelectedPattern = null;
                _toolStripStatusLabel.Text = "You have filtered out all patterns. Please change some of them.";
            }
            else
            {
                _buttonNext.Enabled = true;
                if(_bformLoaded)
                    _toolStripStatusLabel.Text = string.Format("There are {0} design patterns to choose from", patterns.Count);
            }
        }


        private void _dataGridViewParams_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            DataGridView dgv = sender as DataGridView;
            ITemplateArgument tmpl = dgv.Rows[e.RowIndex].DataBoundItem as ITemplateArgument;
            tmpl.Value = dgv.Rows[e.RowIndex].Cells["Value"].FormattedValue.ToString();
        }

        private void _buttonConfig_Click(object sender, EventArgs e)
        {
            ConfigForm cfg = new ConfigForm();
            if (cfg.ShowDialog(this) == DialogResult.OK)
                ShowColumns();
        }

        private void _dataGridViewPatterns_MouseUp(object sender, MouseEventArgs e)
        {
            // Load context menu on right mouse click
            DataGridView dgv = sender as DataGridView;
            if (dgv != null && e.Button == MouseButtons.Right)
            {
                DataGridView.HitTestInfo hitTestInfo = dgv.HitTest(e.X, e.Y);
                switch (hitTestInfo.Type)
                {
                    case DataGridViewHitTestType.Cell:
                        _contextMenuStrip.Tag = hitTestInfo;
                        _contextMenuStrip.Items[0].Visible = true;
                        _contextMenuStrip.Items[0].Text = string.Format("Filter {0} to '{1}'", dgv.Columns[hitTestInfo.ColumnIndex].Name,
                            dgv.Rows[hitTestInfo.RowIndex].Cells[hitTestInfo.ColumnIndex].Value);
                        _contextMenuStrip.Items[0].Enabled = _dataGridViewPatterns.Columns[hitTestInfo.ColumnIndex].Tag != null;
                        _contextMenuStrip.Items[1].Text = string.Empty;
                        _contextMenuStrip.Items[1].Visible = false;
                        _contextMenuStrip.Items[2].Text = "Remove all Filters";
                        _contextMenuStrip.Show(dgv, new Point(e.X, e.Y));
                        break;
                    case DataGridViewHitTestType.ColumnHeader:
                        _contextMenuStrip.Tag = dgv.Columns[hitTestInfo.ColumnIndex];
                        _contextMenuStrip.Items[0].Text = string.Empty;
                        _contextMenuStrip.Items[0].Visible = false;
                        _contextMenuStrip.Items[1].Visible = true;
                        _contextMenuStrip.Items[1].Text = string.Format("Hide Column '{0}'", dgv.Columns[hitTestInfo.ColumnIndex].Name);
                        //don't hide 'PatternName'
                        _contextMenuStrip.Items[1].Enabled = string.Compare(dgv.Columns[hitTestInfo.ColumnIndex].Name, "PatternName", true) != 0;
                        _contextMenuStrip.Items[2].Text = "Show all Columns";
                        _contextMenuStrip.Show(dgv, new Point(e.X, e.Y));
                        break;

                    case DataGridViewHitTestType.None:
                        _contextMenuStrip.Tag = hitTestInfo;
                        _contextMenuStrip.Items[0].Visible = false;
                        _contextMenuStrip.Items[0].Text = string.Empty;
                        _contextMenuStrip.Items[1].Text = string.Empty;
                        _contextMenuStrip.Items[1].Visible = false;
                        _contextMenuStrip.Items[2].Text = "Remove all Filters";
                        _contextMenuStrip.Show(dgv, new Point(e.X, e.Y));
                        break;
                    default:
                        MessageBox.Show(this, string.Format("Type {0}", hitTestInfo.Type));
                        break;

                }
            }
        }

        private void _contextMenuStrip_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (e.ClickedItem == _contextMenuStrip.Items[0])
            {
                DataGridView.HitTestInfo hitTestInfo = e.ClickedItem.Owner.Tag as DataGridView.HitTestInfo;
                DataGridViewColumn column = _dataGridViewPatterns.Columns[hitTestInfo.ColumnIndex];
                column.Visible = false;
                ComboBox cbx = column.Tag as ComboBox;
                if (cbx != null)
                {
                    cbx.SelectedItem = _dataGridViewPatterns.Rows[hitTestInfo.RowIndex].Cells[hitTestInfo.ColumnIndex].Value;
                }


            }
            if (e.ClickedItem == _contextMenuStrip.Items[1])
            {
                DataGridViewColumn column = e.ClickedItem.Owner.Tag as DataGridViewColumn;
                column.Visible = false;
                ComboBox cbx = column.Tag as ComboBox;
                if (cbx != null)
                {
                    cbx.SelectedItem = "Any";
                    cbx.Enabled = false;
                }
                ColumnsEnum colEnum = (ColumnsEnum)Enum.Parse(typeof(ColumnsEnum), column.Name, true);
                BaseLanguageProvider.Columns ^= colEnum;


            }
            if (e.ClickedItem == _contextMenuStrip.Items[2])
            {
                DataGridView.HitTestInfo hitTestInfo = e.ClickedItem.Owner.Tag as DataGridView.HitTestInfo;
                if (hitTestInfo == null)
                {
                    ColumnsEnum[] allColumns = ProjectUtils.GetEnumValues<ColumnsEnum>();
                    BaseLanguageProvider.Columns = allColumns.Aggregate((acc, loc) => loc | acc);//restore all columns
                }
                else
                {
                    for (int i = 0; i < _dataGridViewPatterns.Columns.Count; i++ )
                    {
                        ComboBox cbx = _dataGridViewPatterns.Columns[i].Tag as ComboBox;
                        if (cbx == null)
                            continue;
                        cbx.SelectedItem = "Any";
                        cbx.Enabled = true;

                    }
                }
                ShowColumns();
            }
        }

        private void _buttonAbout_Click(object sender, EventArgs e)
        {
            FormAbout fa = new FormAbout();
            fa.ShowDialog(this);
        }

    }
}
